@@ -22,10 +22,13 @@ gint session_last = -1;
 gsize session_number = 0;
 gboolean had_error, paused, channel_closed, first_load_groups = TRUE;
 gchar *argv_session, *error_list;
+gboolean cmdline_session = FALSE, dryrunning = FALSE;
 GtkWidget *main_window, *rsync = NULL;
 gchar config_command[MAXPATH];
-gboolean config_output = FALSE, config_remember = TRUE, config_errorlist = TRUE;
+gboolean config_output = FALSE, config_remember = TRUE, config_errorlist = TRUE, config_log = FALSE;
 GTimeVal startime, pausedtime;
+gchar *grsync_dir, *rsync_help = NULL, *rsync_man = NULL;
+FILE *log_file = NULL;
 
 
 gboolean get_checkbox(void* main, gchar* name) {
@@ -51,8 +54,9 @@ void load_settings(GtkWidget* main, gchar* session) {
 	GError *error_handler = NULL;
 	GKeyFile *settings_file;
 	gchar *stmp;
+	gboolean btmp;
 
-	g_sprintf(settings_file_path, "%s/.grsync/%s", g_get_home_dir(), "grsync.ini");
+	g_sprintf(settings_file_path, "%s/%s", grsync_dir, "grsync.ini");
 	// initializes config_command before other functions which may use it
 	strcpy(config_command, "rsync");
 
@@ -64,14 +68,24 @@ void load_settings(GtkWidget* main, gchar* session) {
 
 	set_main_title(session, NULL);
 	settings_file = g_key_file_new();
-	g_key_file_load_from_file(settings_file, settings_file_path, 	G_KEY_FILE_NONE, &error_handler);
+	g_key_file_load_from_file(settings_file, settings_file_path, 	G_KEY_FILE_NONE, NULL);
 
 	// load configuration (preferences)
 	stmp = g_key_file_get_string(settings_file, CONFIG_GROUP, "command", NULL);
 	if (stmp != NULL) strncpy(config_command, stmp, MAXPATH - 1);
-	config_output = g_key_file_get_boolean(settings_file, CONFIG_GROUP, "output", NULL);
-	config_remember = g_key_file_get_boolean(settings_file, CONFIG_GROUP, "remember", NULL);
-	config_errorlist = g_key_file_get_boolean(settings_file, CONFIG_GROUP, "errorlist", NULL);
+	
+	btmp = g_key_file_get_boolean(settings_file, CONFIG_GROUP, "output", &error_handler);
+	if (error_handler != NULL) g_clear_error (&error_handler);
+	else config_output = btmp;
+	btmp = g_key_file_get_boolean(settings_file, CONFIG_GROUP, "remember", &error_handler);
+	if (error_handler != NULL) g_clear_error (&error_handler);
+	else config_remember = btmp;
+	btmp = g_key_file_get_boolean(settings_file, CONFIG_GROUP, "errorlist", &error_handler);
+	if (error_handler != NULL) g_clear_error (&error_handler);
+	else config_errorlist = btmp;
+	btmp = g_key_file_get_boolean(settings_file, CONFIG_GROUP, "logging", &error_handler);
+	if (error_handler != NULL) g_clear_error (&error_handler);
+	else config_log = btmp;
 
 	// load specific session
 	stmp = g_key_file_get_string(settings_file, session, "text_source", NULL);
@@ -86,6 +100,15 @@ void load_settings(GtkWidget* main, gchar* session) {
 	gtk_entry_set_text((GtkEntry*) lookup_widget((GtkWidget*) main, "entry_additional"), stmp);
 	g_free(stmp);
 
+	stmp = g_key_file_get_string(settings_file, session, "text_com_before", NULL);
+	if (stmp == NULL) stmp = g_strconcat("", NULL);
+	gtk_entry_set_text((GtkEntry*) lookup_widget((GtkWidget*) main, "entry_com_before"), stmp);
+	g_free(stmp);
+	stmp = g_key_file_get_string(settings_file, session, "text_com_after", NULL);
+	if (stmp == NULL) stmp = g_strconcat("", NULL);
+	gtk_entry_set_text((GtkEntry*) lookup_widget((GtkWidget*) main, "entry_com_after"), stmp);
+	g_free(stmp);
+
 	set_checkbox(main, "check_time", g_key_file_get_boolean(settings_file, session, "check_time", NULL));
 	set_checkbox(main, "check_perm", g_key_file_get_boolean(settings_file, session, "check_perm", NULL));
 	set_checkbox(main, "check_owner", g_key_file_get_boolean(settings_file, session, "check_owner", NULL));
@@ -106,13 +129,14 @@ void load_settings(GtkWidget* main, gchar* session) {
 	set_checkbox(main, "check_compr", g_key_file_get_boolean(settings_file, session, "check_compr", NULL));
 	set_checkbox(main, "check_backup", g_key_file_get_boolean(settings_file, session, "check_backup", NULL));
 
+	set_checkbox(main, "check_com_before", g_key_file_get_boolean(settings_file, session, "check_com_before", NULL));
+	set_checkbox(main, "check_com_after", g_key_file_get_boolean(settings_file, session, "check_com_after", NULL));
+
 	g_key_file_free (settings_file);
-	g_clear_error (&error_handler);
 }
 
 
 void save_settings(GtkWidget* main, gchar *session) {
-	gchar settings_file_path[MAXPATH];
 	gchar settings_file_name[MAXPATH];
 	gchar *key_data;
 	FILE *key_file;
@@ -120,11 +144,10 @@ void save_settings(GtkWidget* main, gchar *session) {
 	GKeyFile *settings_file;
 	const gchar *stmp;
 
-	g_sprintf(settings_file_path, "%s/.grsync", g_get_home_dir());
-	g_sprintf(settings_file_name, "%s/%s", settings_file_path, "grsync.ini");
+	g_sprintf(settings_file_name, "%s/%s", grsync_dir, "grsync.ini");
 
 //	g_printf(_("Saving settings to %s...\n"), settings_file_name);
-	g_mkdir(settings_file_path, 0700);
+	g_mkdir(grsync_dir, 0700);
 	settings_file = g_key_file_new();
 	g_key_file_load_from_file(settings_file, settings_file_name, G_KEY_FILE_NONE, NULL);
 
@@ -133,6 +156,7 @@ void save_settings(GtkWidget* main, gchar *session) {
 	g_key_file_set_boolean(settings_file, CONFIG_GROUP, "output", config_output);
 	g_key_file_set_boolean(settings_file, CONFIG_GROUP, "remember", config_remember);
 	g_key_file_set_boolean(settings_file, CONFIG_GROUP, "errorlist", config_errorlist);
+	g_key_file_set_boolean(settings_file, CONFIG_GROUP, "logging", config_log);
 	if (session_last != -1) g_key_file_set_string(settings_file, CONFIG_GROUP, "last_session", groups[session_last]);
 
 	// save specific session
@@ -142,6 +166,10 @@ void save_settings(GtkWidget* main, gchar *session) {
 	g_key_file_set_string(settings_file, session, "text_dest", stmp);
 	stmp = gtk_entry_get_text((GtkEntry*) lookup_widget((GtkWidget*) main, "entry_additional"));
 	g_key_file_set_string(settings_file, session, "text_addit", stmp);
+	stmp = gtk_entry_get_text((GtkEntry*) lookup_widget((GtkWidget*) main, "entry_com_before"));
+	g_key_file_set_string(settings_file, session, "text_com_before", stmp);
+	stmp = gtk_entry_get_text((GtkEntry*) lookup_widget((GtkWidget*) main, "entry_com_after"));
+	g_key_file_set_string(settings_file, session, "text_com_after", stmp);
 
 	g_key_file_set_boolean(settings_file, session, "check_time", get_checkbox(main, "check_time"));
 	g_key_file_set_boolean(settings_file, session, "check_perm", get_checkbox(main, "check_perm"));
@@ -163,6 +191,9 @@ void save_settings(GtkWidget* main, gchar *session) {
 	g_key_file_set_boolean(settings_file, session, "check_compr", get_checkbox(main, "check_compr"));
 	g_key_file_set_boolean(settings_file, session, "check_backup", get_checkbox(main, "check_backup"));
 
+	g_key_file_set_boolean(settings_file, session, "check_com_before", get_checkbox(main, "check_com_before"));
+	g_key_file_set_boolean(settings_file, session, "check_com_after", get_checkbox(main, "check_com_after"));
+
 	key_data = g_key_file_to_data(settings_file, NULL, &error_handler);
 	key_file = fopen(settings_file_name, "w");
 	if (key_file != NULL) {
@@ -184,7 +215,7 @@ int compare_strings (char **p1, char **p2) {
 
 gboolean load_groups(GtkWidget* main, gchar *session) {
 	gchar settings_file_path[MAXPATH], *stmp;
-	g_sprintf(settings_file_path, "%s/.grsync/%s", g_get_home_dir(), "grsync.ini");
+	g_sprintf(settings_file_path, "%s/%s", grsync_dir, "grsync.ini");
 
 	if (g_file_test(settings_file_path, G_FILE_TEST_EXISTS)) {
 		GError *error_handler = NULL;
@@ -201,7 +232,7 @@ gboolean load_groups(GtkWidget* main, gchar *session) {
 		g_key_file_load_from_file(settings_file, settings_file_path, 	G_KEY_FILE_NONE, &error_handler);
 		groups = g_key_file_get_groups(settings_file, &session_number);
 
-		if (first_load_groups && g_key_file_get_boolean(settings_file, CONFIG_GROUP, "remember", NULL)) {
+		if (!cmdline_session && first_load_groups && g_key_file_get_boolean(settings_file, CONFIG_GROUP, "remember", NULL)) {
 			stmp = g_key_file_get_string(settings_file, CONFIG_GROUP, "last_session", NULL);
 			if (stmp != NULL) {
 				session = stmp;
@@ -252,7 +283,7 @@ gboolean load_groups(GtkWidget* main, gchar *session) {
 void show_browse_source(GtkButton *button, gpointer user_data) {
 	GtkWidget *dialog;
 	gint retval;
-	dialog = gtk_file_chooser_dialog_new (_("Browse"), NULL, GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER,
+	dialog = gtk_file_chooser_dialog_new (_("Browse"), (GtkWindow*) main_window, GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER,
 		GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT, NULL);
 	gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (dialog),
 		gtk_entry_get_text((GtkEntry*) lookup_widget((GtkWidget*) button, "text_source")));
@@ -273,7 +304,7 @@ void show_browse_source(GtkButton *button, gpointer user_data) {
 void show_browse_dest(GtkButton *button, gpointer user_data) {
 	GtkWidget *dialog;
 	gint retval;
-	dialog = gtk_file_chooser_dialog_new (_("Browse"), NULL, GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER,
+	dialog = gtk_file_chooser_dialog_new (_("Browse"), (GtkWindow*) main_window, GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER,
 		GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT, NULL);
 	gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (dialog),
 		gtk_entry_get_text((GtkEntry*) lookup_widget((GtkWidget*) button, "text_dest")));
@@ -300,7 +331,7 @@ void on_play_clicked(GtkButton *button, gpointer user_data) {
 	argv[i++] = config_command;
 	argv[i++] = "-r";
 
-	if (user_data != NULL && strcmp(user_data, "dryrun") == 0) argv[i++] = "-n";
+	if (dryrunning) argv[i++] = "-n";
 	if (get_checkbox(button, "check_time")) argv[i++] = "-t";
 	if (get_checkbox(button, "check_perm")) argv[i++] = "-p";
 	if (get_checkbox(button, "check_owner")) argv[i++] = "-o";
@@ -334,26 +365,69 @@ void on_play_clicked(GtkButton *button, gpointer user_data) {
 	argv[i++] = NULL;
 
 	paused = FALSE;
-	rsync = create_rsync();
-	if (config_output || (user_data != NULL && strcmp(user_data, "dryrun") == 0)) {
+	rsync = create_dialog_rsync();
+	gtk_window_set_transient_for((GtkWindow*) rsync, (GtkWindow*) main_window);
+	if (config_output || dryrunning) {
 		gtk_expander_set_expanded((GtkExpander*) lookup_widget((GtkWidget*) rsync, "expander_rsync"), TRUE);
 	}
 	if (config_errorlist) error_list = g_strconcat("", NULL);
-		
+	if (config_log) {
+		gtmp = g_strconcat(grsync_dir, "/", groups[session_last], ".log", NULL);
+		log_file = fopen(gtmp, "w");
+		g_free(gtmp);
+	}
+
 	gtk_widget_show(rsync);
 	gtk_window_set_title((GtkWindow*)rsync, "rsync: running");
+	if (dryrunning) gtk_label_set_text((GtkLabel*)lookup_widget((GtkWidget*) rsync, "label_file"), _("Simulation in progress"));
 }
 
+
 void on_dryrun_clicked (GtkButton *button, gpointer user_data) {
-	on_play_clicked(button, "dryrun");
+	dryrunning = TRUE;
+	on_play_clicked(button, NULL);
+}
+
+
+void set_tooltip(GtkTooltips *tooltips, gchar *long_opt, gchar *widget_str) {
+	gchar *tmpc, *tmpc2, *out, *opt;
+	GtkWidget *widget;
+	GtkTooltipsData *oldtip;
+
+	opt = rsync_man != NULL ? g_strconcat(long_opt, "\n", NULL) : g_strconcat(long_opt, " ", NULL);
+	out = rsync_man != NULL ? g_strconcat(rsync_man, NULL) : g_strconcat(rsync_help, NULL);
+	tmpc = g_strrstr(out, opt);
+//	tmpc = g_strstr_len(out, strlen(out), opt);
+
+	if (tmpc != NULL && *tmpc) {
+		while (*tmpc != '\n') tmpc--;
+		tmpc++;
+		tmpc2 = rsync_man != NULL ? strstr(tmpc, "\n\n\n") : strchr(tmpc, '\n');
+		*tmpc2 = '\0';
+		g_strstrip(tmpc);
+
+		widget = (GtkWidget*) lookup_widget((GtkWidget*) main_window, widget_str);
+		oldtip = gtk_tooltips_data_get(widget);
+
+		if (oldtip != NULL) tmpc2 = g_strconcat(tmpc, "\n\n", oldtip->tip_text, NULL);
+		else tmpc2 = g_strconcat(tmpc, NULL);
+		gtk_tooltips_set_tip(GTK_TOOLTIPS(tooltips), widget, tmpc2, "");
+		g_free(tmpc2);
+	}
+	g_free(opt);
+	g_free(out);
 }
 
 
 void on_main_create(GtkWindow *window, gpointer user_data) {
 	GtkWidget *error;
+	gchar *stmp, *tmpc;
+	GtkTooltips *rsync_tips;
+
+	grsync_dir = g_strconcat(g_get_home_dir(), "/.grsync", NULL);
 
 	if (!load_groups((GtkWidget*) window, argv_session)) {
-		error = gtk_message_dialog_new(NULL, GTK_DIALOG_DESTROY_WITH_PARENT, GTK_MESSAGE_ERROR, GTK_BUTTONS_CANCEL,
+		error = gtk_message_dialog_new(window, GTK_DIALOG_DESTROY_WITH_PARENT, GTK_MESSAGE_ERROR, GTK_BUTTONS_CANCEL,
 			_("The session you specified on the command line doesn't exists"));
 		gtk_dialog_run(GTK_DIALOG(error));
 		gtk_widget_destroy(error);
@@ -363,31 +437,55 @@ void on_main_create(GtkWindow *window, gpointer user_data) {
 	}
 	load_settings((GtkWidget*) window, argv_session);
 
-	if (g_find_program_in_path(config_command) == NULL) {
-		error = gtk_message_dialog_new(NULL, GTK_DIALOG_DESTROY_WITH_PARENT, GTK_MESSAGE_ERROR, GTK_BUTTONS_CANCEL,
+	stmp = g_strconcat(config_command, " --help", NULL);
+	if (!g_spawn_command_line_sync(stmp, &rsync_help, NULL, NULL, NULL)) {
+		error = gtk_message_dialog_new(window, GTK_DIALOG_DESTROY_WITH_PARENT, GTK_MESSAGE_ERROR, GTK_BUTTONS_CANCEL,
 			_("Unable to execute the rsync command line tool. It is needed by this program. You may need to install the rsync package or make it available in the default executable search PATH. See file/preferences to change the default rsync executable command."));
 		gtk_dialog_run(GTK_DIALOG(error));
 		gtk_widget_destroy(error);
-		return;
+	} else {
+//		g_spawn_command_line_sync("man rsync", &rsync_man, NULL, NULL, NULL);
+		rsync_tips = gtk_tooltips_new();
+		set_tooltip(rsync_tips, "--times", "check_time");
+		set_tooltip(rsync_tips, "--perms", "check_perm");
+		set_tooltip(rsync_tips, "--owner", "check_owner");
+		set_tooltip(rsync_tips, "--group", "check_group");
+		set_tooltip(rsync_tips, "--one-file-system", "check_onefs");
+		set_tooltip(rsync_tips, "--verbose", "check_verbose");
+		set_tooltip(rsync_tips, "--progress", "check_progr");
+		set_tooltip(rsync_tips, "--delete", "check_delete");
+		set_tooltip(rsync_tips, "--ignore-existing", "check_exist");
+		set_tooltip(rsync_tips, "--size-only", "check_size");
+		set_tooltip(rsync_tips, "--checksum", "check_sum");
+		set_tooltip(rsync_tips, "--links", "check_symlink");
+		set_tooltip(rsync_tips, "-D", "check_dev");
+		set_tooltip(rsync_tips, "--existing", "check_update");
+		set_tooltip(rsync_tips, "--partial", "check_keepart");
+		set_tooltip(rsync_tips, "--numeric-ids", "check_mapuser");
+		set_tooltip(rsync_tips, "--compress", "check_compr");
+		set_tooltip(rsync_tips, "--backup", "check_backup");
 	}
+	g_free(stmp);
 }
 
 
-void on_main_destroy (GtkObject *object, gpointer user_data) {
+void on_main_destroy(GtkObject *object, gpointer user_data) {
 	gint sel = gtk_combo_box_get_active((GtkComboBox*) lookup_widget((GtkWidget*) object, "combo_session"));
 	save_settings((GtkWidget*) object, groups[sel]);
 	g_strfreev(groups);
+	if (rsync_help != NULL) g_free(rsync_help);
+	if (rsync_man != NULL) g_free(rsync_man);
 	gtk_main_quit();
 }
 
 
-void on_close_clicked (GtkButton *button, gpointer user_data) {
+void on_close_clicked(GtkButton *button, gpointer user_data) {
 	if (rsync_pid) {
 		if (paused) kill(rsync_pid, SIGCONT);
 		kill(rsync_pid, SIGTERM);
 		gtk_widget_set_sensitive(lookup_widget((GtkWidget*) button, "button_pause"), FALSE);
 	} else {
-		gtk_widget_destroy(lookup_widget((GtkWidget*) button, "rsync"));
+		gtk_widget_destroy(rsync);
 		set_main_title(groups[session_last], NULL);
 		rsync = NULL;
 	}
@@ -398,7 +496,7 @@ void set_global_progress(GtkProgressBar* progr, gdouble fraction) {
 	GTimeVal curtime;
 	glong elapsed, remaining;
 	gchar tmps[50];
-	
+
 	gtk_progress_bar_set_fraction(progr, fraction);
 	g_get_current_time(&curtime);
 	elapsed = curtime.tv_sec - startime.tv_sec;
@@ -406,47 +504,83 @@ void set_global_progress(GtkProgressBar* progr, gdouble fraction) {
 	snprintf(tmps, 49, _("%.f%% (%li:%02li elapsed, %li:%02li remaining)"), round(fraction * 100), elapsed / 60, elapsed % 60,
 		remaining / 60, remaining % 60);
 	gtk_progress_bar_set_text(progr, tmps);
-	
+
 	set_main_title(groups[session_last], tmps);
 }
 
 
+void scroll_to_end(GtkTextView* view) {
+	GtkTextBuffer *buffer;
+	GtkTextIter iter;
+	GtkTextMark *mark;
+
+	buffer = gtk_text_view_get_buffer(view);
+	gtk_text_buffer_get_end_iter(buffer, &iter);
+	mark = gtk_text_buffer_create_mark(buffer, "end_mark", &iter, TRUE);
+	gtk_text_view_scroll_mark_onscreen(view, mark);
+	gtk_text_buffer_delete_mark(buffer, mark);
+}
+
+
 void rsync_cleanup(gpointer data) {
 	gchar tmps[50];
 	GTimeVal curtime;
 	glong elapsed;
 	GtkWidget *errors;
 	GtkTextBuffer *buffer;
+	GtkTextView *view;
 	GtkTextIter iter;
 	GtkTextTag *tag;
+	gchar *comm_str, *comm_out, *comm_err, *comm_after;
 
 	if (channel_closed) {
-		gtk_button_set_label((GtkButton*)lookup_widget((GtkWidget*) data, "close"), "gtk-close");
+		view = (GtkTextView*) lookup_widget((GtkWidget*) data, "textview_output");
 
-		GtkLabel *tmpl = (GtkLabel*)lookup_widget((GtkWidget*) data, "label_file");
+		if (get_checkbox((GtkWidget*) main_window, "check_com_after")) {
+			comm_str = (gchar*) gtk_entry_get_text((GtkEntry*) lookup_widget((GtkWidget*) main_window, "entry_com_after"));
+			if (!g_spawn_command_line_sync(comm_str, &comm_out, &comm_err, NULL, NULL)) {
+				comm_out = g_strconcat("", NULL);
+				comm_err = g_strconcat("Error launching command!\n", NULL);
+			}
+			comm_after = g_strconcat("\n", _("*** Launching AFTER command:\n"), comm_str, "\n", comm_out, comm_err, NULL);
+			g_free(comm_out);
+			g_free(comm_err);
+
+			buffer = gtk_text_view_get_buffer(view);
+			gtk_text_buffer_insert_at_cursor(buffer, comm_after, -1);
+			scroll_to_end(view);
+			if (config_log) fputs(comm_after, log_file);
+			g_free(comm_after);
+		}	
+
+		gtk_button_set_label((GtkButton*) lookup_widget((GtkWidget*) data, "close"), "gtk-close");
+
+		GtkLabel *tmpl = (GtkLabel*) lookup_widget((GtkWidget*) data, "label_file");
 		if (had_error) {
 			gtk_label_set_markup(tmpl, _("<span foreground=\"red\" weight=\"bold\">Completed with errors!</span>"));
 		} else {
 			gtk_label_set_markup(tmpl, _("<span foreground=\"darkgreen\" weight=\"bold\">Completed successfully!</span>"));
-			set_global_progress((GtkProgressBar*)lookup_widget((GtkWidget*) data, "progress_global"), 1);
+			set_global_progress((GtkProgressBar*) lookup_widget((GtkWidget*) data, "progress_global"), 1);
 		}
 		gtk_widget_set_sensitive(lookup_widget((GtkWidget*) data, "button_pause"), FALSE);
-		gtk_widget_set_sensitive(lookup_widget((GtkWidget*) data, "textview_output"), TRUE);
-		gtk_window_set_title((GtkWindow*)rsync, "rsync: stopped");
-		
+		gtk_widget_set_sensitive((GtkWidget*) view, TRUE);
+		gtk_window_set_title((GtkWindow*) rsync, "rsync: stopped");
+
 		g_spawn_close_pid(rsync_pid);
 		rsync_pid = 0;
 		g_free(argv);
-		
+
 		if (config_errorlist && had_error) {
-			errors = create_errors();
-			buffer = gtk_text_view_get_buffer((GtkTextView*)	lookup_widget(errors, "textview_errors"));
+			errors = create_dialog_errors();
+			gtk_window_set_transient_for((GtkWindow*) errors, (GtkWindow*) rsync);
+			buffer = gtk_text_view_get_buffer((GtkTextView*) lookup_widget(errors, "textview_errors"));
 			tag = gtk_text_buffer_create_tag(buffer, "fore-red", "foreground", "red", NULL);
 			gtk_text_buffer_get_start_iter(buffer, &iter);
 			gtk_text_buffer_insert_with_tags_by_name(buffer, &iter, error_list, -1, "fore-red", NULL);
 			gtk_widget_show(errors);
 			g_free(error_list);
 		}
+		dryrunning = FALSE;
 	} else {
 		channel_closed = TRUE;
 	}
@@ -476,38 +610,40 @@ gboolean out_watch(GIOChannel *source, GIOCondition condition, gpointer data) {
 		carriage = strchr(str->str, '\r');
 
 		gtk_text_buffer_insert_at_cursor(buffer, (gchar*)str->str, -1);
-		gtk_text_buffer_get_end_iter(buffer, &iter);
-		gtk_text_view_scroll_to_iter((GtkTextView*)data, &iter, 0, TRUE, 0.0, 1.0);
+		scroll_to_end((GtkTextView*)data);
+		if (config_log) fputs((gchar*)str->str, log_file);
 
-		if (strchr(str->str, '%') && str->str[0] == ' ') {
-			if (oldfile != NULL) file = g_strconcat(oldfile, NULL);
-			else file = g_strconcat(str->str, NULL);
+		if (!dryrunning) {
+			if (strchr(str->str, '%') && str->str[0] == ' ') {
+				if (oldfile != NULL) file = g_strconcat(oldfile, NULL);
+				else file = g_strconcat(str->str, NULL);
+	
+				fraction = g_ascii_strtod(strchr(strchr(str->str, '%') - 4, ' '), NULL) / 100;
+				tmpp = (GtkProgressBar*)lookup_widget((GtkWidget*) data, "progress_file");
+				gtk_progress_bar_set_fraction(tmpp, fraction);
+				snprintf(tmps, 5, "%.f%%", round(fraction * 100));
+				gtk_progress_bar_set_text(tmpp, tmps);
 
-			fraction = g_ascii_strtod(strchr(strchr(str->str, '%') - 4, ' '), NULL) / 100;
-			tmpp = (GtkProgressBar*)lookup_widget((GtkWidget*) data, "progress_file");
-			gtk_progress_bar_set_fraction(tmpp, fraction);
-			snprintf(tmps, 5, "%.f%%", round(fraction * 100));
-			gtk_progress_bar_set_text(tmpp, tmps);
+				fraction = 0;
+				if (strchr(str->str, '%') != strrchr(str->str, '%')) fraction = g_ascii_strtod(strchr(strrchr(str->str, '%') - 6, ' '), NULL) / 100;
+				else if (strchr(str->str, '=')) {
+					total = g_ascii_strtod(strrchr(str->str, '/') + 1, NULL);
+					done = total - g_ascii_strtod(strrchr(str->str, '=') + 1, NULL);
+					fraction = done / total;
+				}
 
-			fraction = 0;
-			if (strchr(str->str, '%') != strrchr(str->str, '%')) fraction = g_ascii_strtod(strchr(strrchr(str->str, '%') - 6, ' '), NULL) / 100;
-			else if (strchr(str->str, '=')) {
-				total = g_ascii_strtod(strrchr(str->str, '/') + 1, NULL);
-				done = total - g_ascii_strtod(strrchr(str->str, '=') + 1, NULL);
-				fraction = done / total;
+				if (fraction) set_global_progress((GtkProgressBar*)lookup_widget((GtkWidget*) data, "progress_global"), fraction);
+			} else {
+				file = g_strconcat(str->str, NULL);
+				if (oldfile != NULL) g_free(oldfile);
+				oldfile = g_strconcat(str->str, NULL);
 			}
-
-			if (fraction) set_global_progress((GtkProgressBar*)lookup_widget((GtkWidget*) data, "progress_global"), fraction);
-		} else {
-			file = g_strconcat(str->str, NULL);
-			if (oldfile != NULL) g_free(oldfile);
-			oldfile = g_strconcat(str->str, NULL);
+			g_strchomp(file);
+			if (file[0] != 0) gtk_label_set_text((GtkLabel*)lookup_widget((GtkWidget*) data, "label_file"), file);
+			g_free(file);
 		}
 
-		g_strchomp(file);
-		if (file[0] != 0) gtk_label_set_text((GtkLabel*)lookup_widget((GtkWidget*) data, "label_file"), file);
 		g_string_free(str, TRUE);
-		g_free(file);
 	} else {
 //	if (condition & G_IO_HUP || condition & G_IO_ERR) {
 		g_io_channel_shutdown (source, FALSE, NULL);
@@ -539,9 +675,14 @@ gboolean err_watch(GIOChannel *source, GIOCondition condition, gpointer data) {
 	//	if (status == G_IO_STATUS_NORMAL) {
 		gtk_text_buffer_get_end_iter(buffer, &iter);
 		gtk_text_buffer_insert_with_tags_by_name(buffer, &iter, (gchar*)str->str, -1, "fore-red", NULL);
-		gtk_text_buffer_get_end_iter(buffer, &iter);
-		gtk_text_view_scroll_to_iter((GtkTextView*)data, &iter, 0, TRUE, 0.0, 1.0);
+		scroll_to_end((GtkTextView*)data);
 	//	}
+
+		if (config_log) {
+			// fputs("*ERROR* ", log_file);
+			fputs((gchar*)str->str, log_file);
+		}
+
 		tmpc = g_strconcat(error_list, str->str, NULL);
 		g_free(error_list);
 		error_list = tmpc;
@@ -565,19 +706,37 @@ void on_rsync_show(GtkWidget *widget, gpointer user_data) {
 	GIOChannel *chout, *cherr;
 	GtkTextTag *tag;
 	GtkWidget *error;
-	gchar *comm_str;
+	gchar *comm_str, *comm_text, *comm_out, *comm_err, *comm_before, *tmpc;
 
 	rsync_pid = 0;
 	had_error = FALSE;
 	gtk_label_set_ellipsize((GtkLabel*)lookup_widget(widget, "label_file"), PANGO_ELLIPSIZE_MIDDLE);
-	comm_str = g_strjoinv(" ", argv);
 	
-	view = (GtkTextView*)lookup_widget((GtkWidget*) widget, "textview_output");
-	buffer = gtk_text_view_get_buffer(view);
-	gtk_text_buffer_insert_at_cursor(buffer, _("Launching rsync command:\n"), -1);
-	gtk_text_buffer_insert_at_cursor(buffer, comm_str, -1);
-	gtk_text_buffer_insert_at_cursor(buffer, "\n\n", -1);
+	if (get_checkbox((GtkWidget*) main_window, "check_com_before")) {
+		comm_str = (gchar*) gtk_entry_get_text((GtkEntry*) lookup_widget((GtkWidget*) main_window, "entry_com_before"));
+		if (!g_spawn_command_line_sync(comm_str, &comm_out, &comm_err, NULL, NULL)) {
+			comm_out = g_strconcat("", NULL);
+			comm_err = g_strconcat("Error launching command!\n", NULL);
+		}
+		comm_before = g_strconcat(_("*** Launching BEFORE command:\n"), comm_str, "\n", comm_out, comm_err, "\n", NULL);
+		g_free(comm_out);
+		g_free(comm_err);
+	} else {
+		comm_before = g_strconcat("", NULL);
+	}
+	
+	comm_str = g_strjoinv(" ", argv);
+	if (dryrunning) tmpc =_("*** Launching RSYNC command (simulation mode):\n");
+	else tmpc = _("*** Launching RSYNC command:\n");
+	comm_text = g_strconcat(comm_before, tmpc, comm_str, "\n\n", NULL);
+	g_free(comm_before);
 	g_free(comm_str);
+
+	view = (GtkTextView*)lookup_widget(widget, "textview_output");
+	buffer = gtk_text_view_get_buffer(view);
+	gtk_text_buffer_insert_at_cursor(buffer, comm_text, -1);
+	if (config_log) fputs(comm_text, log_file);
+	g_free(comm_text);
 //	gtk_widget_get_size_request((GtkWidget*) lookup_widget(widget, "scrolled1"), &bogus, &height_expander);
 	g_get_current_time(&startime);
 
@@ -596,18 +755,22 @@ void on_rsync_show(GtkWidget *widget, gpointer user_data) {
 	} else {
 		gtk_button_set_label((GtkButton*)lookup_widget(widget, "close"), "gtk-close");
 		gtk_widget_set_sensitive(lookup_widget(widget, "button_pause"), FALSE);
-		error = gtk_message_dialog_new(NULL, GTK_DIALOG_DESTROY_WITH_PARENT, GTK_MESSAGE_ERROR, GTK_BUTTONS_CANCEL,
+		error = gtk_message_dialog_new((GtkWindow*) main_window, GTK_DIALOG_DESTROY_WITH_PARENT, GTK_MESSAGE_ERROR, GTK_BUTTONS_CANCEL,
 			_("Cannot execute rsync"));
 		gtk_dialog_run(GTK_DIALOG(error));
 		gtk_widget_destroy(error);
 	}
 }
 
-void on_rsync_destroy (GtkObject *object, gpointer user_data) {
+void on_rsync_destroy(GtkObject *object, gpointer user_data) {
 	if (rsync_pid) {
 		if (paused) kill(rsync_pid, SIGCONT);
 		kill(rsync_pid, SIGTERM);
 	}
+	if (log_file != NULL) {
+		fclose(log_file);
+		log_file = NULL;
+	}
 }
 
 
@@ -626,6 +789,7 @@ void on_session_add_clicked(GtkButton *button, gpointer user_data) {
 	gint result, i = 0;
 
 	add = create_dialog_new_session();
+	gtk_window_set_transient_for((GtkWindow*) add, (GtkWindow*) main_window);
 	result = gtk_dialog_run ((GtkDialog*) add);
 
 	if (result == GTK_RESPONSE_OK) {
@@ -633,7 +797,7 @@ void on_session_add_clicked(GtkButton *button, gpointer user_data) {
 		newses = (gchar*) gtk_entry_get_text((GtkEntry*) lookup_widget((GtkWidget*) add, "entry_session_name"));
 
 		if (strcmp(CONFIG_GROUP, newses) == 0) {
-			confirm = gtk_message_dialog_new(NULL, GTK_DIALOG_DESTROY_WITH_PARENT, GTK_MESSAGE_ERROR,
+			confirm = gtk_message_dialog_new((GtkWindow*) main_window, GTK_DIALOG_DESTROY_WITH_PARENT, GTK_MESSAGE_ERROR,
 				GTK_BUTTONS_CANCEL, _("This session name is reserved, you cannot call it like this"));
 			gtk_dialog_run(GTK_DIALOG(confirm));
 			gtk_widget_destroy(confirm);
@@ -643,7 +807,7 @@ void on_session_add_clicked(GtkButton *button, gpointer user_data) {
 
 		while (groups[i] != NULL && strcmp(groups[i], newses)) i++;
 		if (groups[i] != NULL) {
-			confirm = gtk_message_dialog_new(NULL, GTK_DIALOG_DESTROY_WITH_PARENT, GTK_MESSAGE_ERROR,
+			confirm = gtk_message_dialog_new((GtkWindow*) main_window, GTK_DIALOG_DESTROY_WITH_PARENT, GTK_MESSAGE_ERROR,
 				GTK_BUTTONS_CANCEL, _("A session named like this already exists"));
 			gtk_dialog_run(GTK_DIALOG(confirm));
 			gtk_widget_destroy(confirm);
@@ -660,7 +824,6 @@ void on_session_add_clicked(GtkButton *button, gpointer user_data) {
 
 
 void on_session_del_clicked(GtkButton *button, gpointer user_data) {
-	gchar settings_file_path[MAXPATH];
 	gchar settings_file_name[MAXPATH];
 	gchar *key_data;
 	FILE *key_file;
@@ -671,21 +834,20 @@ void on_session_del_clicked(GtkButton *button, gpointer user_data) {
 
 	i = gtk_combo_box_get_active((GtkComboBox*) lookup_widget((GtkWidget*) button, "combo_session"));
 	if (strcmp(groups[i], "default") == 0) {
-		confirm = gtk_message_dialog_new(NULL, GTK_DIALOG_DESTROY_WITH_PARENT, GTK_MESSAGE_ERROR,
+		confirm = gtk_message_dialog_new((GtkWindow*) main_window, GTK_DIALOG_DESTROY_WITH_PARENT, GTK_MESSAGE_ERROR,
 			GTK_BUTTONS_CANCEL, _("You cannot delete the default session"));
 		gtk_dialog_run(GTK_DIALOG(confirm));
 		gtk_widget_destroy(confirm);
 		return;
 	}
 
-	confirm = gtk_message_dialog_new(NULL, GTK_DIALOG_DESTROY_WITH_PARENT, GTK_MESSAGE_QUESTION,
+	confirm = gtk_message_dialog_new((GtkWindow*) main_window, GTK_DIALOG_DESTROY_WITH_PARENT, GTK_MESSAGE_QUESTION,
 		GTK_BUTTONS_OK_CANCEL, _("Are you sure you want to delete the '%s' session?"), groups[i]);
 	result = gtk_dialog_run(GTK_DIALOG(confirm));
 	gtk_widget_destroy(confirm);
 
 	if (result == GTK_RESPONSE_OK) {
-		g_sprintf(settings_file_path, "%s/.grsync", g_get_home_dir());
-		g_sprintf(settings_file_name, "%s/%s", settings_file_path, "grsync.ini");
+		g_sprintf(settings_file_name, "%s/%s", grsync_dir, "grsync.ini");
 		settings_file = g_key_file_new();
 		g_key_file_load_from_file(settings_file, settings_file_name, G_KEY_FILE_NONE, NULL);
 		g_key_file_remove_group(settings_file, groups[i], &error_handler);
@@ -712,7 +874,7 @@ void on_button_about_clicked(GtkButton *button, gpointer user_data) {
 //	GtkWidget *about = gtk_about_dialog_new();
 	GdkPixbuf *pixbuf;
 	pixbuf = create_pixbuf("grsync.png");
-	gtk_show_about_dialog(NULL, "name", PACKAGE, "version", VERSION, "logo", pixbuf, "comments", _("A GTK GUI for rsync"),
+	gtk_show_about_dialog((GtkWindow*) main_window, "name", PACKAGE, "version", VERSION, "logo", pixbuf, "comments", _("A GTK GUI for rsync"),
 		"copyright", _("(C) Piero Orsoni and others. Released under the GPL.\nSee AUTHORS and COPYING for details"),
 		"website", "http://www.opbyte.it/grsync/", NULL);
 	gdk_pixbuf_unref(pixbuf);
@@ -756,10 +918,12 @@ void on_preferences1_activate(GtkMenuItem *menuitem, gpointer user_data) {
 	gint result;
 
 	config = create_dialog_config();
+	gtk_window_set_transient_for((GtkWindow*) config, (GtkWindow*) main_window);
 	gtk_entry_set_text((GtkEntry*) lookup_widget((GtkWidget*) config, "entry_command"), config_command);
 	set_checkbox(config, "check_output", config_output);
 	set_checkbox(config, "check_remember", config_remember);
 	set_checkbox(config, "check_errorlist", config_errorlist);
+	set_checkbox(config, "check_log", config_log);
 
 	result = gtk_dialog_run((GtkDialog*) config);
 	if (result == GTK_RESPONSE_OK) {
@@ -767,6 +931,7 @@ void on_preferences1_activate(GtkMenuItem *menuitem, gpointer user_data) {
 		config_output = get_checkbox(config, "check_output");
 		config_remember = get_checkbox(config, "check_remember");
 		config_errorlist = get_checkbox(config, "check_errorlist");
+		config_log = get_checkbox(config, "check_log");
 	}
 
 	gtk_widget_destroy(config);
@@ -779,33 +944,10 @@ void on_rsync_info_activate(GtkMenuItem *menuitem, gpointer user_data) {
 
 	tmps = g_strconcat(config_command, " --version", NULL);
 	g_spawn_command_line_sync(tmps, &result, NULL, NULL, NULL);
-	dialog = gtk_message_dialog_new(NULL, GTK_DIALOG_DESTROY_WITH_PARENT, GTK_MESSAGE_INFO, GTK_BUTTONS_CLOSE, result);
+	dialog = gtk_message_dialog_new((GtkWindow*) main_window, GTK_DIALOG_DESTROY_WITH_PARENT, GTK_MESSAGE_INFO, GTK_BUTTONS_CLOSE, result);
 	gtk_dialog_run(GTK_DIALOG(dialog));
 
 	gtk_widget_destroy(dialog);
 	g_free(tmps);
 	g_free(result);
 }
-
-
-gboolean on_main_state_changed(GtkWidget *widget, GdkEvent *event, gpointer user_data) {
-	if (rsync != NULL && event->window_state.changed_mask & GDK_WINDOW_STATE_ICONIFIED) {
-		if (event->window_state.new_window_state & GDK_WINDOW_STATE_ICONIFIED) gtk_window_iconify((GtkWindow*) rsync);
-		else {
-			gtk_window_deiconify((GtkWindow*) rsync);
-			// does nothing, should focus the rsync window over main window
-			// probably because the rsync window is still not deiconified here
-			// gtk_widget_grab_focus(rsync);
-		}
-	}
-  return FALSE;
-}
-
-
-gboolean on_rsync_window_state_event (GtkWidget *widget, GdkEvent *event, gpointer user_data) {
-	if (event->window_state.changed_mask & GDK_WINDOW_STATE_ICONIFIED) {
-		if (event->window_state.new_window_state & GDK_WINDOW_STATE_ICONIFIED) gtk_window_iconify((GtkWindow*) main_window);
-		else gtk_window_deiconify((GtkWindow*) main_window);
-	}
-  return FALSE;
-}
